@@ -1,9 +1,8 @@
-// $Id: svndigest.cc 1586 2015-08-13 15:46:32Z jari $
+// $Id: svndigest.cc 1635 2023-03-30 04:16:57Z peter $
 
 /*
 	Copyright (C) 2006, 2007, 2008, 2009 Jari Häkkinen, Peter Johansson
-	Copyright (C) 2010, 2011, 2012 Peter Johansson
-	Copyright (C) 2015 Jari Häkkinen
+	Copyright (C) 2010, 2011, 2012, 2023 Peter Johansson
 
 	This file is part of svndigest, http://dev.thep.lu.se/svndigest
 
@@ -21,6 +20,8 @@
 	along with svndigest. If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include <config.h>
+
 #include "svndigestParameter.h"
 
 #include "lib/CacheRemover.h"
@@ -68,17 +69,6 @@ void prepare_report(const svndigestParameter& option, const std::string& repo,
 
 int main( int argc, char* argv[])
 {
-	// Need to create an instance of Graph (plstream) since there is a
-	// bug in plplot with dynamic loading of runtime libraries. The
-	// issue is reported in the plplot issue tracking system
-	// https://sf.net/tracker/?func=detail&aid=3009045&group_id=2915&atid=102915
-	// and as http://dev.thep.lu.se/svndigest/ticket/472. Simply remove
-	// this comment and the next line of code when plplot developers
-	// fixes their bug.
-  //
-  // Omitting the dummy instance will cause segmentation fault.
-	Graph dummy_never_used("/dev/null", "null");
-
 	// Reading commandline options
 	svndigestParameter option;
 	try {
@@ -125,7 +115,7 @@ int main( int argc, char* argv[])
 			std::cout << "Parsing directory tree" << std::endl;
 
 		SvndigestVisitor visitor(option.verbose(), option.ignore_cache(),
-														 option.report());
+														 option.report(), option.update());
 		tree.traverse(visitor);
 		NodeCounter file_count;
 		tree.traverse(file_count);
@@ -235,8 +225,7 @@ void prepare_report(const svndigestParameter& option, const std::string& repo,
 	std::string target_path = concatenate_path(option.targetdir(),
 																						 file_name(option.root()));
 	// remove target if needed
-	if (node_exist(target_path)) {
-		assert(option.force());
+	if (node_exist(target_path) && option.force()) {
 		remove_target(target_path, option.verbose());
 	}
 
@@ -253,13 +242,7 @@ void prepare_report(const svndigestParameter& option, const std::string& repo,
 
 	if (option.verbose())
 		std::cout << "Generating output" << std::endl;
-	mkdir(option.root_basename());
+	mkdir_p(option.root_basename());
 	chdir(option.root_basename());
 	print_css("svndigest.css");
-	// create structure StatsType/Author/LineType
-	/*
-	for (std::map<std::string, Stats*>::const_iterator i(stats.stats().begin());
-			 i!=stats.stats().end(); ++i)
-		create_file_struct(i->first+std::string("/"), *i->second);
-	*/
 }
