@@ -1,10 +1,9 @@
-// $Id: SVN.cc 1574 2015-02-18 18:08:32Z jari $
+// $Id: SVN.cc 1635 2023-03-30 04:16:57Z peter $
 
 /*
 	Copyright (C) 2006 Jari Häkkinen
 	Copyright (C) 2007, 2008 Jari Häkkinen, Peter Johansson
-	Copyright (C) 2010, 2011, 2012 Peter Johansson
-	Copyright (C) 2015 Jari Häkkinen
+	Copyright (C) 2010, 2012, 2015, 2023 Peter Johansson
 
 	This file is part of svndigest, http://dev.thep.lu.se/svndigest
 
@@ -22,8 +21,12 @@
 	along with svndigest. If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include <config.h>
+
 #include "SVN.h"
 
+#include "utility.h"
+
 #include <map>
 #include <string>
 #include <vector>
@@ -41,6 +44,7 @@
 
 #include <string>
 
+#include <iostream>
 namespace theplu {
 namespace svndigest {
 
@@ -55,6 +59,7 @@ namespace svndigest {
 			msg_ = svn_err_best_message(error_, buf, bufsize);
 		}
 		if (msg.size()) {
+			msg_ += ". ";
 			msg_ += msg;
 		}
 	}
@@ -151,6 +156,7 @@ namespace svndigest {
 		// Set up svn administration area access. The whole structure is
 		// setup, maybe this is unnecessary?
 		const char* canonical_path=svn_path_internal_style(path.c_str(), pool_);
+		abs_wc_root_path_ = absolute_path(canonical_path);
 		if ((err=svn_wc_adm_open3(&adm_access_, NULL, canonical_path,
 															false, -1, context_->cancel_func,
 															context_->cancel_baton, pool_))){
@@ -160,12 +166,15 @@ namespace svndigest {
 		}
 
 		// get a session to the repository
-		struct root_url_receiver_baton rurb;
-		client_info(path, root_url_receiver, static_cast<void*>(&rurb));
+		struct url_receiver_baton urb;
+		client_info(path, url_receiver, static_cast<void*>(&urb));
 		if ((err=svn_client_open_ra_session(&ra_session_,
-																				rurb.path.c_str(),
+																				urb.root_url.c_str(),
 																				context_, pool_)))
 			cleanup_failed_init(err, "SVN: svn_client_open_ra_session failed");
+
+		// extract and store URL relative to root URL
+		relative_url_ = relative_path(urb.url, urb.root_url);
 	}
 
 
@@ -180,27 +189,15 @@ namespace svndigest {
 	}
 
 
-	void SVN::cleanup(svn_error_t *err,apr_pool_t *pool,
-										std::string message, bool mute)
-	{
-		svn_error_clear(err);
-		if (pool){
-			svn_pool_destroy(pool);
-			pool=NULL;
-		}
-		assert(message.size()); // compatible with r1213
-		if (!mute)
-			throw SVNException(message, err);
-		// mute implies we don't wanna hear the message from svn_error_t
-		throw SVNException(message);
-	}
-
-
 	void SVN::cleanup_failed_init(svn_error_t *err, const std::string& message)
 	{
 		assert(message.size()); // compatible with r1213
-		cleanup(err,pool_, message, true);
-		assert(false && "cleanup should throw");
+		svn_error_clear(err);
+		if (pool_) {
+			svn_pool_destroy(pool_);
+			pool_=NULL;
+		}
+		throw SVNException(message);
 	}
 
 
@@ -234,27 +231,53 @@ namespace svndigest {
 		peg.kind=svn_opt_revision_unspecified;
 		start.kind=svn_opt_revision_number;
 		start.value.number=0;
-		apr_pool_t *subpool = svn_pool_create(pool_);
-		svn_error_t* err=svn_client_blame3(path.c_str(), &peg, &start, &head,
-																			 svn_diff_file_options_create(subpool),
-																			 false, receiver, baton, context_,
-																			 subpool);
+		AprPool subpool(pool_);
+		svn_error_t* err =
+			svn_client_blame3(path.c_str(), &peg, &start, &head,
+												svn_diff_file_options_create(subpool.get()),
+												false, receiver, baton, context_,
+												subpool.get());
 		if (err)
-			// cleanup will throw an exception
-			cleanup(err, subpool, "SVN::client_blame: svn_client_blame3 failed");
-		svn_pool_destroy(subpool);
+			throw SVNException("SVN::client_blame: svn_client_blame3 failed", err);
+	}
+
+
+	void SVN::client_cat(const std::string& path, svn_revnum_t rev,
+											 std::string& result)
+	{
+		AprPool subpool(pool_);
+		AprPool scratchpool(pool_);
+		svn_stringbuf_t* str = svn_stringbuf_create("", subpool.get());
+		assert(str);
+		svn_stream_t* out = svn_stream_from_stringbuf(str, subpool.get());
+		assert(out);
+
+		svn_error_t* err = NULL;
+
+		svn_opt_revision_t peg;
+		peg.kind = svn_opt_revision_unspecified;
+		svn_opt_revision_t revision;
+		revision.kind = svn_opt_revision_number;
+		revision.value.number = rev;
+
+		err = svn_client_cat3(NULL, out, path.c_str(), &peg, &revision,
+													false, context_, subpool.get(), scratchpool.get());
+
+		if (err)
+			throw SVNException("client_cat failed", err);
+
+		// copy to result
+		result = std::string(str->data, str->len);
 	}
 
 
 	void SVN::client_info(const std::string& path, svn_info_receiver_t receiver,
 												void *baton)
 	{
-		apr_pool_t *subpool = svn_pool_create(pool_);
+		AprPool subpool(pool_);
 		if (svn_error_t *err=svn_client_info(path.c_str(), NULL, NULL, receiver,
-																				 baton, false, context_, subpool))
-			// cleanup will throw an exception
-			cleanup(err, subpool, "repository: svn_client_info failed");
-		svn_pool_destroy(subpool);
+																				 baton, false, context_,subpool.get()))
+			throw SVNException("repository: svn_client_info failed", err);
 	}
 
 
@@ -262,11 +285,12 @@ namespace svndigest {
 											 svn_log_message_receiver_t receiver, void *baton)
 	{
 		// Allocate space in subpool to pool_ for apr_path (here a string).
-		apr_pool_t *subpool = svn_pool_create(pool_);
-		apr_array_header_t* apr_path=apr_array_make(subpool,1,4);
+		AprPool subpool(pool_);
+		apr_array_header_t* apr_path=apr_array_make(subpool.get(),1,4);
 		// Copy path to apr_path.
 		(*((const char **) apr_array_push(apr_path))) =
-			apr_pstrdup(subpool, svn_path_internal_style(path.c_str(),subpool));
+			apr_pstrdup(subpool.get(),
+									svn_path_internal_style(path.c_str(), subpool.get()));
 
 		// Setup to retrieve all commit logs.
 		svn_opt_revision_t peg, start, head;
@@ -277,10 +301,8 @@ namespace svndigest {
 									svn_opt_revision_head : svn_opt_revision_base );
 		svn_error_t* err=NULL;
 		if ((err=svn_client_log3(apr_path, &peg, &start, &head, 0, false, false,
-														 receiver, baton, context_, subpool)))
-			// cleanupp will throw an exception
-			cleanup(err, subpool, "commit_dates: svn_client_log3 failed");
-		svn_pool_destroy(subpool);
+														 receiver, baton, context_, subpool.get())))
+			throw SVNException("commit_dates: svn_client_log3 failed", err);
 	}
 
 
@@ -291,34 +313,33 @@ namespace svndigest {
 		peg.kind=svn_opt_revision_unspecified;
 		revision.kind = ( svn_path_is_url(path.c_str()) ?
 											svn_opt_revision_head : svn_opt_revision_base );
-		apr_pool_t *subpool = svn_pool_create(pool_);
+		AprPool subpool(pool_);
 		apr_array_header_t * properties;
 		svn_error_t *err=svn_client_proplist2(&properties, path.c_str(), &peg,
-																					&revision, false, context_, subpool);
+																					&revision, false, context_,
+																					subpool.get());
 		if (err)
-			cleanup(err, subpool, "repository: svn_client_proplist2 failed");
+			throw SVNException("repository: svn_client_proplist2 failed", err);
 
     for (int j = 0; j < properties->nelts; ++j) {
       svn_client_proplist_item_t *item =
         ((svn_client_proplist_item_t **)properties->elts)[j];
-      for (apr_hash_index_t *hi = apr_hash_first(subpool, item->prop_hash); hi;
-           hi = apr_hash_next (hi)) {
+      for (apr_hash_index_t *hi =apr_hash_first(subpool.get(),item->prop_hash);
+					 hi; hi = apr_hash_next (hi)) {
         const void *key;
         void *val;
         apr_hash_this (hi, &key, NULL, &val);
 				svn_string_t *value;
 				err=svn_subst_detranslate_string(&value,
 																				 static_cast<const svn_string_t*>(val),
-																				 false, subpool);
+																				 false, subpool.get());
 				if (err)
-					cleanup(err, subpool,
-									path +
+					throw SVNException(path +
 									" property: svn_subst_detranslate_string failed on key " +
-									static_cast<const char*>(key));
+														 static_cast<const char*>(key), err);
 				property[static_cast<const char*>(key)]=value->data;
       }
 		}
-		svn_pool_destroy(subpool);
 	}
 
 
@@ -339,17 +360,19 @@ namespace svndigest {
 	}
 
 
-	svn_error_t* SVN::root_url_receiver(void *baton, const char *,
-																			const svn_info_t *info, apr_pool_t*)
+	svn_error_t* SVN::url_receiver(void *baton, const char *,
+																 const svn_info_t *info, apr_pool_t*)
 	{
 		if (!info)
 			throw SVNException(std::string("SVN::url_receriver: ") +
 												 "Failed to acquire an svn info object");
 
-		root_url_receiver_baton* rurb=
-			static_cast<struct root_url_receiver_baton*>(baton);
+		url_receiver_baton* rurb=
+			static_cast<struct url_receiver_baton*>(baton);
 		if (info->repos_root_URL)
-			rurb->path=info->repos_root_URL;
+			rurb->root_url = info->repos_root_URL;
+		if (info->URL)
+			rurb->url = info->URL;
 		return SVN_NO_ERROR;
 	}
 
@@ -357,13 +380,13 @@ namespace svndigest {
 	SVN::vc_status SVN::version_controlled(const std::string& path)
 	{
 		svn_wc_status2_t* status=NULL;
-		apr_pool_t *subpool = svn_pool_create(pool_);
+		AprPool subpool(pool_);
 		if (svn_error_t *err=
-				svn_wc_status2(&status,svn_path_internal_style(path.c_str(), subpool),
-											 adm_access_, subpool))
-			// cleanup will throw an exception
-			cleanup(err, subpool,"version_controlled(): svn_config_get_config failed");
-		svn_pool_destroy(subpool);
+				svn_wc_status2(&status,
+											 svn_path_internal_style(path.c_str(), subpool.get()),
+											 adm_access_, subpool.get()))
+			throw SVNException("version_controlled(): svn_config_get_config failed",
+												 err);
 
 		if ((status->text_status==svn_wc_status_none) ||
 				(status->text_status==svn_wc_status_unversioned))
@@ -374,4 +397,25 @@ namespace svndigest {
 		return unresolved;
 	}
 
+
+	SVN::AprPool::AprPool(apr_pool_t* pool)
+		: pool_(NULL)
+	{
+		pool_ = svn_pool_create(pool);
+		assert(pool_);
+	}
+
+
+	SVN::AprPool::~AprPool(void)
+	{
+		if (pool_)
+			svn_pool_destroy(pool_);
+	}
+
+
+	apr_pool_t* SVN::AprPool::get(void)
+	{
+		return pool_;
+	}
+
 }} // end of namespace svndigest and namespace theplu
