@@ -1,8 +1,8 @@
 #!/bin/sh
 
-# $Id: cmd_format_test.sh 1220 2010-10-13 04:10:52Z peter $
+# $Id: cmd_format_test.sh 1532 2012-10-05 01:14:42Z peter $
 #
-# Copyright (C) 2010 Peter Johansson
+# Copyright (C) 2010, 2012 Peter Johansson
 #
 # This file is part of svndigest, http://dev.thep.lu.se/svndigest
 #
@@ -21,7 +21,7 @@
 
 required="repo"
 
-. ./init.sh || exit 1
+. test/init.sh || exit 99
 set -e
 
 SVNDIGEST_run 1 -f --format=banana
