@@ -1,7 +1,7 @@
 #!/bin/sh
-# $Id: config2_test.sh 1220 2010-10-13 04:10:52Z peter $
+# $Id: config2_test.sh 1532 2012-10-05 01:14:42Z peter $
 #
-# Copyright (C) 2009, 2010 Peter Johansson
+# Copyright (C) 2009, 2010, 2012 Peter Johansson
 #
 # This file is part of svndigest, http://dev.thep.lu.se/svndigest
 #
@@ -23,7 +23,7 @@
 # svndigest's config file is not shipped
 required=repo
 
-. ./init.sh || exit 1
+. test/init.sh || exit 99
 set -e
 
 SVNDIGEST_run 0 -g --config-file $abs_top_srcdir/.svndigest/config || exit_fail
