@@ -1,7 +1,7 @@
 #!/bin/sh
-# $Id: config3_test.sh 1220 2010-10-13 04:10:52Z peter $
+# $Id: config3_test.sh 1532 2012-10-05 01:14:42Z peter $
 #
-# Copyright (C) 2010 Peter Johansson
+# Copyright (C) 2010, 2012 Peter Johansson
 #
 # This file is part of svndigest, http://dev.thep.lu.se/svndigest
 #
@@ -20,7 +20,7 @@
 
 # test that we can use /dev/null as config file
 
-. ./init.sh || exit 1
+. test/init.sh || exit 99
 set -e
 
 test -e /dev/null || exit_skip
