@@ -1,7 +1,7 @@
 #!/bin/sh
-# $Id: permission_test.sh 1392 2011-07-12 02:56:29Z peter $
+# $Id: permission_test.sh 1532 2012-10-05 01:14:42Z peter $
 #
-# Copyright (C) 2011 Peter Johansson
+# Copyright (C) 2011, 2012 Peter Johansson
 #
 # This file is part of svndigest, http://dev.thep.lu.se/svndigest
 #
@@ -20,7 +20,7 @@
 
 required="repo"
 
-. ./init.sh || exit 1
+. test/init.sh || exit 1
 set -e
 
 $SVN revert toy_project -R
