@@ -1,8 +1,8 @@
-// $Id: ColumnStream.cc 2919 2012-12-19 06:54:23Z peter $
+// $Id: ColumnStream.cc 3330 2014-10-14 08:03:25Z peter $
 
 /*
 	Copyright (C) 2007, 2008 Jari Häkkinen, Peter Johansson
-	Copyright (C) 2010, 2011, 2012 Peter Johansson
+	Copyright (C) 2010, 2011, 2012, 2014 Peter Johansson
 
 	This file is part of yat, http://dev.thep.lu.se/yat
 
@@ -27,9 +27,9 @@
 #include <cassert>
 #include <ostream>
 
-namespace theplu{
-namespace yat{
-namespace utility{
+namespace theplu {
+namespace yat {
+namespace utility {
 
 	ColumnStream::ColumnStream(std::ostream& os, size_t columns)
 		: activated_(0), margins_(columns), os_(os), width_(columns, 8)
@@ -55,10 +55,8 @@ namespace utility{
 
 	void ColumnStream::fill(size_t column, size_t count)
 	{
-		while(count<width_[column]){
-			os_ << ' ';
-			++count;
-		}
+		if (count<width_[column])
+			os_ << std::string(width_[column]-count, ' ');
 	}
 
 
@@ -78,20 +76,20 @@ namespace utility{
 	}
 
 
-	size_t& ColumnStream::margin(size_t c) 
-	{ 
-		return margins_[c]; 
+	size_t& ColumnStream::margin(size_t c)
+	{
+		return margins_[c];
 	}
 
 
 	void ColumnStream::next_column(void)
 	{
 		++activated_;
-		if (activated_>=columns()) {
+		if (activated_>=columns())
 			flush();
-		}
 	}
 
+
 	void ColumnStream::print(std::stringstream& ss)
 	{
 		assert(buffer_[activated_]);
@@ -108,16 +106,16 @@ namespace utility{
 				flush();
 				activated_=0;
 			}
-			else 
+			else
 				*(buffer_[activated_]) << c;
-		ss.get(c);
+			ss.get(c);
 		}
 	}
 
 
-	size_t& ColumnStream::width(size_t c) 
-	{ 
-		return width_[c]; 
+	size_t& ColumnStream::width(size_t c)
+	{
+		return width_[c];
 	}
 
 
@@ -140,11 +138,11 @@ namespace utility{
 					os_ << word;
 					count += word.size();
 					word = "";
-				}				
+				}
 				else {
 					if (!buffer_[column]->good())
 						buffer_[column]->clear(std::ios::goodbit);
-					
+
 					// if line is empty and word is longer than column width, we
 					// have to split the word
 					if (!count) {
@@ -162,11 +160,9 @@ namespace utility{
 					return true;
 				}
 			}
-
-		} 
+		}
 		fill(column, count);
 		return false;
 	}
 
-
 }}} // end of namespace utility, yat, and theplu
