@@ -1,9 +1,10 @@
-// $Id: CommandLine.cc 2881 2012-11-18 01:28:05Z peter $
+// $Id: CommandLine.cc 4252 2022-11-18 02:54:04Z peter $
 
 /*
 	Copyright (C) 2007 Jari Häkkinen, Peter Johansson, Markus Ringnér
 	Copyright (C) 2008 Jari Häkkinen, Peter Johansson
-	Copyright (C) 2009, 2010, 2011, 2012 Peter Johansson
+	Copyright (C) 2009, 2010, 2011, 2012, 2013, 2018 Peter Johansson
+	Copyright (C) 2022 Jari Häkkinen, Peter Johansson
 
 	This file is part of the yat library, http://dev.thep.lu.se/yat
 
@@ -105,22 +106,18 @@ namespace utility {
 
 
 	void CommandLine::parse(int argc, char* argv[])
-	{		
+	{
 		parsed_=true;
 		using namespace std;
 		// just in case it is not pristine
-		for (auto& option : options_)
-			option->reset();
+		for_each(options_.begin(), options_.end(),std::mem_fn(&Option::reset));
 
-		std::vector<std::string> arguments;
-		arguments.reserve(argc);
-		for (int i=0; i<argc; ++i)
-			arguments.push_back(argv[i]);
-		std::vector<std::string>::iterator arg(arguments.begin());		
-		std::vector<std::string>::iterator end(arguments.end());		
-		stringstream ss(*arg++);
+		std::vector<std::string> arguments(argv, argv+argc);
+		std::vector<std::string>::iterator arg(arguments.begin());
+		std::vector<std::string>::iterator end(arguments.end());
 		// keeping string after last /
-		while (getline(ss, program_name_,'/')) {}
+		program_name_ = basename(*arg);
+		++arg;
 
 		bool ok=true;
 		std::string error_message;
@@ -137,8 +134,8 @@ namespace utility {
 		}
 		// validate all options
 		try {
-			for (const auto& option : options_)
-				option->validate();
+			for_each(options_.begin(), options_.end(),
+							 std::mem_fn(&Option::validate));
 		}
 		catch (cmd_error& e) {
 			if (ok) { // we only store first error
@@ -146,14 +143,14 @@ namespace utility {
 				error_message = e.what();
 			}
 		}
-		
+
 		if (!ok) {
 			std::stringstream ss;
 			ss << program_name_ << ": " << error_message;
 			throw cmd_error(ss.str());
 		}
 	}
-	
+
 
 	void CommandLine::parse_long(std::vector<std::string>::iterator& arg,
 															 std::vector<std::string>::iterator& last)
@@ -166,21 +163,21 @@ namespace utility {
 		if (!value.empty()){
 			*arg = value;
 			*(--arg) = std::string("--")+key;
-		}					
+		}
 		else
 			*arg = key;
 		std::map<std::string, Option*>::const_iterator
 			iter(long_options_.find(key));
 		if (iter!=long_options_.end())
 			iter->second->parse(arg, last);
-		else if (key.size()>3 && key.substr(0,3)=="no-") { 
+		else if (key.size()>3 && key.substr(0,3)=="no-") {
 			iter = long_options_.find(key.substr(3));
 			if (iter!=long_options_.end())
 				iter->second->parse(arg, last);
-		}						
+		}
 		if (iter==long_options_.end()) {
 			std::stringstream ss3;
-			ss3 << "unrecognized option `" << key << "'\n" << try_help();
+			ss3 << "unrecognized option '" << key << "'\n" << try_help();
 			throw cmd_error(ss3.str());
 		}
 	}
@@ -197,8 +194,8 @@ namespace utility {
 				ss2 << ": invalid option -- " << (*arg)[i] << "\n"
 						<< try_help();
 				throw cmd_error(ss2.str());
-			}				
-			else 
+			}
+			else
 				iter->second->parse(arg, last);
 		}
 	}
@@ -216,7 +213,7 @@ namespace utility {
 		}
 	}
 
-	
+
 	void CommandLine::parse(std::vector<std::string>::iterator& first,
 													std::vector<std::string>::iterator& last)
 	{
@@ -228,7 +225,7 @@ namespace utility {
 			parse_free_arg(first, last);
 
 	}
-	
+
 
 	bool CommandLine::parsed(void) const
 	{
@@ -250,7 +247,7 @@ namespace utility {
 
 	std::string CommandLine::try_help(void) const
 	{
-		return std::string("Try `"+program_name()+" --help' for more information.");
+		return std::string("Try '"+program_name()+" --help' for more information.");
 	}
 
 
@@ -259,26 +256,26 @@ namespace utility {
 		os << cmd.description_ << "\n";
 		ColumnStream cs2(os, 2);
 		std::string::size_type width = 0;
-		for (std::vector<Option*>::const_iterator i(cmd.options_.begin()); 
+		for (std::vector<Option*>::const_iterator i(cmd.options_.begin());
 				 i!=cmd.options_.end();++i) {
 			std::stringstream ss((*i)->print());
 			std::string str;
 			getline(ss, str, '\t');
-			width = std::max(width, str.size()+3);				
+			width = std::max(width, str.size()+3);
 		}
 		cs2.width(0)=width;
 		cs2.width(1)=76-width;
 		cs2.margin(0)=2;
 
-		for (std::vector<Option*>::const_iterator i(cmd.options_.begin()); 
-				 i!=cmd.options_.end();++i) 
+		for (std::vector<Option*>::const_iterator i(cmd.options_.begin());
+				 i!=cmd.options_.end();++i)
 			cs2 << (*i)->print() << "\n";
 
 		return os;
 	}
 
 
-	bool CommandLine::OptionCompare::operator()(const Option* lhs, 
+	bool CommandLine::OptionCompare::operator()(const Option* lhs,
 																							const Option* rhs) const
 	{
 		assert(lhs);
