@@ -1,7 +1,7 @@
-// $Id: Exception.cc 2919 2012-12-19 06:54:23Z peter $
+// $Id: Exception.cc 4207 2022-08-26 04:36:28Z peter $
 
 /*
-	Copyright (C) 2010, 2011, 2012 Peter Johansson
+	Copyright (C) 2010, 2011, 2012, 2020, 2022 Peter Johansson
 
 	This file is part of the yat library, http://dev.thep.lu.se/yat
 
@@ -32,18 +32,23 @@ namespace theplu {
 namespace yat {
 namespace utility {
 
+	invalid_argument::invalid_argument(const std::string& message)
+		: std::invalid_argument(message)
+	{}
+
+
 	runtime_error::runtime_error(std::string message)
-		: std::runtime_error(message) 
+		: std::runtime_error(message)
 	{}
 
 
 	cmd_error::cmd_error(std::string message)
-		: runtime_error(message) 
+		: runtime_error(message)
 	{}
 
 
 	errno_error::errno_error(std::string message)
-		: runtime_error(message + strerror(errno)) 
+		: runtime_error(message + strerror(errno))
 	{}
 
 
