@@ -1,10 +1,12 @@
 #ifndef theplu_yat_utility_segment
 #define theplu_yat_utility_segment
 
-// $Id: Segment.h 1640 2023-04-14 23:55:55Z peter $
+// $Id: Segment.h 4339 2023-04-15 10:35:22Z peter $
 
 /*
-	Copyright (C) 2010 Peter Johansson
+	Copyright (C) 2010, 2015, 2016, 2017 Peter Johansson
+	Copyright (C) 2019 Jari Häkkinen
+	Copyright (C) 2021 Peter Johansson
 
 	This file is part of the yat library, http://dev.thep.lu.se/yat
 
@@ -39,7 +41,7 @@ namespace utility {
 		 [begin, end). Type T must be comparable, optionally with a
 		 comparator Compare, which should implement <a
 		 href="http://www.sgi.com/tech/stl/StrictWeakOrdering.html">
-		 Strict Weak Ordering.</a>.		 
+		 Strict Weak Ordering.</a>.
 
 		 \since new in yat 0.7
 	 */
@@ -65,6 +67,16 @@ namespace utility {
 		Segment(const T& begin, const T& end)
 			: begin_(begin), end_(end) {}
 
+		/**
+			 \brief Constructor
+
+			 Creates a segment [begin, end)
+
+			 \since New in yat 0.19
+		 */
+		Segment(T&& begin, T&& end)
+			: begin_(std::move(begin)), end_(std::move(end)) {}
+
 		/**
 			 \return reference to first element in Segment
 		 */
@@ -84,16 +96,70 @@ namespace utility {
 			 \return const reference to first element greater than Segment
 		 */
 		const T& end(void) const { return end_; }
-		
+
 	private:
 		T begin_;
 		T end_;
-		
+
 		// using compiler generated copying
 		//Segment(const Segment&);
 		//Segment& operator=(const Segment&);
 	};
 
+
+	/**
+		 \return Segment<T>
+
+		 \since ney in yat 0.15
+	 */
+	template<typename T>
+	Segment<T> make_segment(T first, T last)
+	{
+		return Segment<T>(first, last);
+	}
+
+
+	/**
+		 \brief Inequality operator
+
+		 Inequality operator is defined via the operator< of T or in the
+		 general case Compare.
+
+		 \return true if Compare returns true for either when comparing
+		 lhs.begin() and rhs.begin() or when comparing lhs.end() and
+		 rhs.end().
+
+		 \relates Segment
+
+		 \since New in yat 0.14
+	*/
+	template<typename T, class Compare>
+	bool operator!=(const Segment<T, Compare>& lhs,
+									const Segment<T, Compare>& rhs)
+	{
+		Compare comp;
+		return comp(lhs.begin(), rhs.begin()) ||	comp(rhs.begin(), lhs.begin())
+			|| comp(lhs.end(), rhs.end()) || comp(rhs.end(), lhs.end());
+	}
+
+
+	/**
+		 \return true if Compare returns false when comparing lhs.begin()
+		 and rhs.begin() as well as when comparing lhs.end() and
+		 rhs.end().
+
+		 \relates Segment
+
+		 \since New in yat 0.14
+	*/
+	template<typename T, class Compare>
+	bool operator==(const Segment<T, Compare>& lhs,
+									const Segment<T, Compare>& rhs)
+	{
+		return !(lhs!=rhs);
+	}
+
+
 	/**
 		 This function takes two Segment and compare them with comparator
 		 Compare. If all elements in \a lhs is less than all elements in
@@ -135,7 +201,7 @@ namespace utility {
 		 \since new in yat 0.7
 	 */
 	template<typename T, class Compare>
-	int compare_3way(const Segment<T, Compare>& lhs, 
+	int compare_3way(const Segment<T, Compare>& lhs,
 									 const Segment<T, Compare>& rhs)
 	{
 		if (compare(lhs, rhs))
@@ -154,7 +220,7 @@ namespace utility {
 		 \since new in yat 0.7
 	 */
 	template<typename T, class Compare>
-	int compare_3way(const T& element, 
+	int compare_3way(const T& element,
 									 const Segment<T, Compare>& segment)
 	{
 		Compare comp;
@@ -165,6 +231,20 @@ namespace utility {
 		return 1;
 	}
 
+	/**
+		 \return -1 if \a all elements in \a segment are less than \a element, 0 if if \a element overlaps with \a segment, and 1 otherwise.
+
+		 \relates Segment
+
+		 \since new in yat 0.14
+	 */
+	template<typename T, class Compare>
+	int compare_3way(const Segment<T, Compare>& segment,
+									 const T& element)
+	{
+		return -compare_3way(element, segment);
+	}
+
 	/**
 		 \return intersection of \a a and \a b. If \a a and \a b do not
 		 overlap an empty Section is returned (begin==end), but the exact
@@ -175,7 +255,7 @@ namespace utility {
 		 \since new in yat 0.7
 	 */
 	template<typename T, class Compare>
-	Segment<T, Compare> intersection(const Segment<T, Compare>& a, 
+	Segment<T, Compare> intersection(const Segment<T, Compare>& a,
 																	 const Segment<T, Compare>& b)
 	{
 		Compare comp;
@@ -183,22 +263,85 @@ namespace utility {
 
 		result.begin() = std::max(a.begin(), b.begin(), comp);
 		// the first max is needed in case a and b don't overlap
-		result.end() = std::max(result.begin(), 
+		result.end() = std::max(result.begin(),
 														std::min(a.end(), b.end(), comp),
 														comp);
 		return result;
 	}
 
+
+	/**
+		 \brief union of two segments
+
+		 \return union of \a a and \a b. If the union of \a a and \a b is
+		 not a single segment, i.e., a.end() < b.begin() and b.end() <
+		 a.begin(), the result is undefined.
+
+		 \relates Segment
+
+		 \see SegmentSet
+
+		 \since new in yat 0.15
+	 */
+	template<typename T, class Compare>
+	Segment<T, Compare> set_union(const Segment<T, Compare>& a,
+																const Segment<T, Compare>& b)
+	{
+		Compare comp;
+		return Segment<T, Compare>(std::min(a.begin(), b.begin(), comp),
+															 std::max(a.end(), b.end(), comp));
+	}
+
+
+	/**
+		 Return true if (and only if) lhs.begin is not less than rhs.begin
+		 and rhs.begin is not less than lhs.end
+
+		 \return true if \a lhs is inluded in \a rhs
+
+		 \since New in yat 0.15
+	 */
+	template<typename T, class Compare>
+	bool includes(const Segment<T, Compare>& lhs,
+								const Segment<T, Compare>& rhs)
+	{
+		Compare comp;
+		return !(comp(lhs.begin(), rhs.begin()) || comp(rhs.end(), lhs.end()));
+	}
+
+
+	/**
+		 Return true if \a lhs is not less than \a rhs and \a rhs is not
+		 less than \a lhs
+
+		 \return true if \a lhs and \a rhs overlap
+
+		 \since New in yat 0.15
+	*/
+	template<typename T, class Compare>
+	bool overlap(const Segment<T, Compare>& lhs, const Segment<T, Compare>& rhs)
+	{
+		return !compare(lhs, rhs) && !compare(rhs, lhs);
+	}
+
+
 	/**
 		 \brief functor using compare
 	 */
 	template<typename T, class Compare>
 	struct SegmentCompare
 	{
+		/// \c first_argument_type is \c Segment<T,Compare>
+		typedef Segment<T,Compare> first_argument_type;
+		/// \c second_argument_type is \c Segment<T,Compare>
+		typedef Segment<T,Compare> second_argument_type;
+		/// \c result_type is \c bool
+		typedef bool result_type;
+
 		/**
 			 \see compare(const Segment<T,Compare>&, const Segment<T,Compare>&)
 		 */
-		bool operator()(const Segment<T, Compare>& lhs, 
+		bool operator()(const Segment<T, Compare>& lhs,
 										const Segment<T, Compare>& rhs) const
 		{ return compare(lhs, rhs); }
 	};
