@@ -1,13 +1,13 @@
 #ifndef _theplu_yat_utility_utility_
 #define _theplu_yat_utility_utility_
 
-// $Id: utility.h 2943 2013-01-04 06:38:38Z peter $
+// $Id: utility.h 4262 2023-01-10 00:05:51Z peter $
 
 /*
 	Copyright (C) 2005 Jari Häkkinen, Peter Johansson, Markus Ringnér
 	Copyright (C) 2006 Jari Häkkinen
 	Copyright (C) 2007, 2008 Jari Häkkinen, Peter Johansson
-	Copyright (C) 2009, 2010, 2011, 2012, 2013 Peter Johansson
+	Copyright (C) 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022 Peter Johansson
 
 	This file is part of the yat library, http://dev.thep.lu.se/yat
 
@@ -31,19 +31,23 @@
 /// @brief Some useful functions are placed here
 ///
 
+#include "config_public.h"
+
 #include "deprecate.h"
 #include "Exception.h"
 #include "yat_assert.h"
 
-#include <gsl/gsl_math.h>
-
 #include <algorithm>
 #include <cctype>
 #include <cmath>
+#include <cstdlib>
+#include <cstring>
 #include <functional>
 #include <limits>
 #include <locale>
 #include <istream>
+#include <iostream>
+#include <numeric>
 #include <string>
 #include <stdexcept>
 #include <sstream>
@@ -54,6 +58,7 @@ namespace theplu {
 namespace yat {
 namespace utility {
 
+
 	/**
 		 Same as posix C function with same name but works on \c
 		 std::string rather than \c char*.
@@ -66,20 +71,6 @@ namespace utility {
 	 */
 	std::string basename(const std::string& fn);
 
-	/**
-		 For each element in resulting range assign it to 0.0 if
-		 corresponding element in input range is NaN else assign it to
-		 1.0.
-
-		 \return true if there is at least one NaN in input range
-		 [first, last).
-
-		 \since New in yat 0.5
-	*/
-	template<typename InputIterator, typename OutputIterator>
-	bool binary_weight(InputIterator first, InputIterator last,
-										 OutputIterator result);
-
 	/**
 		 Same as C function \c chdir but throws on failure (instead of
 		 retrning non-zero).
@@ -105,7 +96,11 @@ namespace utility {
 	/**
 		 \brief convert T to a string
 
-		 T is supposed to be a numerical type.
+		 \c T is supposed to be a numerical type.
+
+		 \see
+		 <a href="http://www.cplusplus.com/reference/string/to_string/">std::to_string</a>
+		 in C++11
 
 		 \since new in yat 0.8
 	 */
@@ -120,6 +115,19 @@ namespace utility {
 	template<typename T>
 	T convert(const std::string& s);
 
+	/**
+		 \brief try to convert
+
+		 \see is<T>
+		 \see convert<T>
+
+		 \return true if conversion was successful
+
+		 \since new in yat 0.15
+	 */
+	template<typename T>
+	bool convert_try(const std::string& s, T& t);
+
 	/**
 		 @brief Copy file \a source to \a target.
 
@@ -128,6 +136,7 @@ namespace utility {
 	*/
 	void copy_file(const std::string& source, const std::string& target);
 
+
 	/**
 		 Same as posix C function with same name but works on \c
 		 std::string rather than \c char*.
@@ -153,6 +162,15 @@ namespace utility {
 	bool fnmatch(const std::string& pattern, const std::string& str,
 							 int flags=0);
 
+	/**
+		 \brief current directory
+
+		 \see ​https://linux.die.net/man/3/getcwd
+
+		 \since New in yat 0.17
+	 */
+	std::string getcwd(void);
+
 	/**
 		 \brief check if string is convertible to (numerical) type \c T
 
@@ -161,14 +179,6 @@ namespace utility {
 	template<typename T>
 	bool is(const std::string& s);
 
-	///
-	/// @return true if string is a double
-	///
-	/// \deprecated Provided for backward compatibility with the 0.4
-	/// API. Use is<double>(const std::string&)
-	///
-	bool is_double(const std::string&) YAT_DEPRECATE;
-
 	/**
 		 This function takes the first word (separated by whitespace) in
 		 \a s, replaces all upper case with lower case, and compares it
@@ -180,23 +190,7 @@ namespace utility {
 	bool is_equal(std::string s, std::string other);
 
 	///
-	/// @return true if string is a float
-	///
-	/// \deprecated Provided for backward compatibility with the 0.4
-	/// API. Use is<float>(const std::string&)
-	///
-	bool is_float(const std::string&) YAT_DEPRECATE;
-
-	///
-	/// @return true if string is an int
-	///
-	/// \deprecated Provided for backward compatibility with the 0.4
-	/// API. Use is<int>(const std::string&)
-	///
-	bool is_int(const std::string&) YAT_DEPRECATE;
-
-	///
-	/// @return true if string is "nan" (case-insensitive) 
+	/// @return true if string is "nan" (case-insensitive)
 	///
 	bool is_nan(const std::string& s);
 
@@ -204,20 +198,20 @@ namespace utility {
 		 The std::istream will be interpreted as outlined here:
 
 		 Lines are separated by character \a line_sep and rows are
-		 separated by character \a sep. 
-		 
+		 separated by character \a sep.
+
 		 The first line is read into a stringstream, which is used to
 		 load the first vector (vec[0]) with elements using
 		 load(stringstream, vec[0], sep).
-		 
+
 		 Therefore, column elements separation has two modes depending
 		 on the value of \a sep.
-		 
+
 		 - If \a sep is the default '\\0' value then column elements are
 		 separated with white space characters except the new line
 		 character. Multiple sequential white space characters are treated
 		 as one separator.
-		 
+
 		 - Setting \a sep to something else than the default value will
 		 change the behaviour to use the \a sep character as the separator
 		 between column elements. Multiple sequential \a sep characters
@@ -236,7 +230,7 @@ namespace utility {
 		 \since New in yat 0.6
 	 */
 	template<typename T>
-	void load(std::istream& is, std::vector<std::vector<T> >& vec, char sep='\0', 
+	void load(std::istream& is, std::vector<std::vector<T> >& vec, char sep='\0',
 						char line_sep='\n', bool ignore_empty=false, bool rectangle=true);
 
 	/**
@@ -244,17 +238,17 @@ namespace utility {
 
 		 Element separation has two modes depending on the value of \a
 		 sep.
-		 
+
 		 - If \a sep is the default '\\0' value then elements are
 		 separated with white space characters. Multiple sequential white
 		 space characters are treated as one separator.
-		 
+
 		 - Setting \a sep to something else than the default value will
 		 change the behaviour to use the \a sep character as the
 		 separator between column elements. Multiple sequential \a sep
 		 characters will be treated as separating elements with missing
 		 values. Missing values are set to std::numeric_limits<T>::quiet_NaN
-		 
+
 		 \note Requirement on T: utility::convert<T> must be supported
 		 (from yat 0.7 T=string is also supported)
 
@@ -264,30 +258,74 @@ namespace utility {
 	void load(std::istream& is, std::vector<T>& vec, char sep='\0');
 
 	/**
-		 \return base-2 logarithm of x
+		 \brief create a directory \a dir
+
+		 \see http://linux.die.net/man/3/mkdir
+
+		 \throw runtime_error if creation failed
 
 		 \since New in yat 0.10
 	 */
-	// c++11 provides std::log2 so perhaps we should call that one if
-	// availalable (but a bit tricky since this is a public header)
-	template<typename T>
-	T log2(T x) { return std::log(x)/M_LN2; }
+	void mkdir(const std::string& dir, mode_t mode=0777);
+
+	/**
+		 Similar to mkdir(const std::string&, mode_t).
+
+		 No error if \a dir already exist. Make parent directories as needed.
+
+		 \since New in yat 0.10
+	 */
+	void mkdir_p(const std::string& dir, mode_t mode=0777);
+
+	/**
+		 \brief recursively print exception what
+
+		 Print \c what() of \a error to \a out. If \a error is nested, the
+		 exception that is nested in \a error is passed to print_what().
+
+		 \note If nested exception is null, function calls terminate (this
+		 behaviour might change in the future).
+
+		 \since new in yat 0.16
+	 */
+	void print_what(const std::exception& error, std::ostream& out=std::cerr);
+
+	/**
+		 same as C function remove but throws errno_error at failure
+
+		 \see http://linux.die.net/man/3/remove
+
+		 \since New in yat 0.12
+	 */
+	void remove(const std::string& fn);
+
+	/**
+		 same as C function with same name but throws errno_error if error
+		 is encountered
+
+		 \see http://linux.die.net/man/3/rename
+
+		 \since New in yat 0.12
+	 */
+	void rename(const std::string& from, const std::string& to);
+
+	/**
+		 same as C function symlink but throws errno_error at failure
+
+		 Creates a symbolic link called \a path2 that contains the string
+		 pointed to by \a path1
+
+		 \see http://linux.die.net/man/3/symlink
+
+		 \since New in yat 0.16
+	 */
+	void symlink(const std::string& path1, const std::string& path2);
+
+/// \cond IGNORE_DOXYGEN
 
 // private namespace
 namespace detail {
 
-	/**
-		 \brief convert s to t
-
-		 used in function is<T> and convert<T>
-
-		 \return true if conversion was successful
-
-		 \internal
-	 */
-	template<typename T>
-	bool convert(const std::string& s, T& t);
-
 	/**
 		 Functor used in load function
 	 */
@@ -300,11 +338,11 @@ namespace detail {
 			 \internal
 		 */
 		void operator()(const std::string& element, std::vector<T>& vec)
-		{ 
+		{
 			if (!element.size())
 				vec.push_back(std::numeric_limits<T>::quiet_NaN());
 			else {
-				vec.push_back(theplu::yat::utility::convert<T>(element));
+				vec.push_back(convert<T>(element));
 			}
 		}
 	};
@@ -321,75 +359,23 @@ namespace detail {
 			 push element on vec's back
 		 */
 		void operator()(const std::string& element, std::vector<std::string>& vec)
-		{ 
+		{
 			vec.push_back(element);
 		}
+
+		void operator()(std::string&& element, std::vector<std::string>& vec)
+		{
+			vec.push_back(std::move(element));
+		}
 	};
 
+
 } // end of namespace detail
 
-
-	/**
-		 \brief create a directory \a dir
-
-		 \see http://linux.die.net/man/3/mkdir
-
-		 \throw runtime_error if creation failed
-	 */
-	void mkdir(const std::string& dir, mode_t mode=0777);
-
-	/**
-		 Similar to mkdir(const std::string&, mode_t).
-
-		 No error if \a dir already exist. Make parent directories as needed.
-	 */
-	void mkdir_p(const std::string& dir, mode_t mode=0777);
-
-	/**
-		 same as C function remove but throws errno_error at failure
-
-		 \see http://linux.die.net/man/3/remove
-	 */
-	void remove(const std::string& fn);
-
-	/**
-		 same as C function with same name but throws errno_error if error
-		 is encountered
-
-		 \see http://linux.die.net/man/3/rename
-
-		 \since New in yat 0.10
-	 */
-	void rename(const std::string& from, const std::string to);
-
-	/**
-		 In \a full_str replace every sub-string \a old_str with \a
-		 new_str;
-	 */
-	void replace(std::string& full_str, std::string old_str, std::string new_str);
+/// \endcond
 
 	// template implementations
 
-	template<typename InputIterator, typename OutputIterator>
-	bool binary_weight(InputIterator first, InputIterator last, 
-										 OutputIterator result)
-	{
-		bool nan=false;
-		while (first!=last) {
-			if (std::isnan(*first)) {
-				*result=0;
-				nan=true;
-			}
-			else
-				*result = 1.0;
-			++first;
-			++result;
-		}
-		return nan;
-	}
-
-
-	// template implementations
 	template<typename T>
 	std::string convert(T input)
 	{
@@ -403,7 +389,7 @@ namespace detail {
 	T convert(const std::string& s)
 	{
 		T result;
-		if (!detail::convert(s, result))
+		if (!convert_try(s, result))
 			throw runtime_error(std::string("yat::utility::convert(\"")+s+
 													std::string("\")"));
 		return result;
@@ -414,13 +400,13 @@ namespace detail {
 	bool is(const std::string& s)
 	{
 		T tmp;
-		return detail::convert(s, tmp);
+		return convert_try(s, tmp);
 	}
 
 
 	template<typename T>
-	void load(std::istream& is, std::vector<std::vector<T> >& matrix, 
-						char sep, char line_sep, bool ignore_empty, 
+	void load(std::istream& is, std::vector<std::vector<T> >& matrix,
+						char sep, char line_sep, bool ignore_empty,
 						bool rectangle)
 	{
 		size_t nof_columns=0;
@@ -428,24 +414,25 @@ namespace detail {
 		while(getline(is, line, line_sep)){
 			if (line.empty() && ignore_empty)
 				continue;
-			matrix.push_back(std::vector<T>());
-			std::vector<T>& v=matrix.back();
+			std::vector<T> v;
 			v.reserve(nof_columns);
-			std::stringstream ss(line);
+			bool empty_last_element = !line.empty() && line.back()==sep;
+			std::stringstream ss(std::move(line));
 			load(ss, v, sep);
 			// add NaN for final separator (or empty string if T=std::string)
 			detail::VectorPusher<T> pusher;
-			if(sep!='\0' && !line.empty() && line[line.size()-1]==sep) 
+			if (empty_last_element && sep!='\0')
 				pusher("", v);
-			
+
 			if (rectangle && nof_columns && v.size()!=nof_columns) {
 				std::ostringstream s;
 				s << "load stream error: "
 					<< "line " << matrix.size() << " has " << v.size()
 					<< " columns; expected " << nof_columns	<< " columns.";
 				throw utility::IO_error(s.str());
-			}				
+			}
 			nof_columns = std::max(nof_columns, v.size());
+			matrix.push_back(std::move(v));
 		}
 
 		// manipulate the state of the stream to be good
@@ -456,21 +443,21 @@ namespace detail {
 	void load(std::istream& is, std::vector<T>& vec, char sep)
 	{
 		detail::VectorPusher<T> pusher;
-		std::string element;
 		while(true) {
+			std::string element;
 			if(sep=='\0')
 				is>>element;
 			else
 				getline(is, element, sep);
-			if(!is.good())
+			if(is.fail())
 				break;
-			pusher(element, vec);
+			pusher(std::move(element), vec);
 		}
 	}
 
-namespace detail {
+
 	template<typename T>
-	bool convert(const std::string& s, T& result)
+	bool convert_try(const std::string& s, T& result)
 	{
 		if (!std::numeric_limits<T>::is_signed) {
 			// first non-whitespace character
@@ -485,7 +472,7 @@ namespace detail {
 		std::istringstream ss(s);
 		ss >> result;
 		if (ss.fail()) {
-			if (is_nan(s)) {
+			if (is_nan(s) || is_equal(s, "-nan")) {
 				result = std::numeric_limits<T>::quiet_NaN();
 				return true;
 			}
@@ -506,7 +493,6 @@ namespace detail {
 		ss >> b;
 		return b.empty();
 	}
-} // of namespace detail
 
 }}} // of namespace utility, yat, and theplu
 
